program gentest

   use dal

   implicit none

   type(DataSetT) :: flatSet, outSet
   type(ArrayT) :: flatArray, outArray, qualArray
   type(TableT) :: outTable
   type(ColumnT) :: pcol

   integer(kind=int32), pointer :: icol(:)
   integer(kind=int8), pointer :: qual(:,:)
   real(kind=single), pointer :: image(:,:)
   real(kind=single), pointer :: flatfieldp(:,:)
   real(kind=single), dimension(0:2047, 0:2047) :: flatfield
   integer :: nx, ny
   integer :: binax1, binax2
   integer :: dims(2)
   logical twixel
   integer x0,y0

   integer(kind=int32) :: allocErr                                   ! Allocate error status.

   type(DataSetT) :: thxDataSetH                                     ! Tracking History Data Auxiliary dataset handle.
   type(TableT) :: thxTableH                                         ! Tracking History Data Auxiliary table handle.
   type(ColumnT), dimension(3) :: thxColumnH                         ! Tracking History Data Auxiliary column handle.

   integer(kind=int16) :: nFrames                                    ! Number of tracking frames.
   type trackingFrame
      real(kind=single) :: dX                                        ! Delta x.
      real(kind=single) :: dY                                        ! Delta y.
      real(kind=single) :: quality                                   ! Quality.
   end type trackingFrame
   type(trackingFrame), allocatable, dimension(:) :: tf              ! Tracking frame array.

   integer(kind=int32), dimension(:), pointer :: dXP                 ! DX pointer.
   integer(kind=int32), dimension(:), pointer :: dYP                 ! DY pointer.

   real(kind=single) :: rShiftX                                      ! Relative shift in x.
   real(kind=single) :: rShiftY                                      ! Relative shift in y.
   integer(kind=int32) :: iShiftX                                    ! Incremental shift in x.
   integer(kind=int32) :: iShiftY                                    ! Incremental shift in y.
   integer(kind=int32) :: iXMin                                      ! Incremental minimum x value for window.
   integer(kind=int32) :: iXMax                                      ! Incremental maximum x value for window.
   integer(kind=int32) :: iYMin                                      ! Incremental minimum y value for window.
   integer(kind=int32) :: iYMax                                      ! Incremental maximum y value for window.

   real, allocatable :: newimage(:,:)
   integer :: i
   integer :: j
   integer :: k
   integer :: l

   call random_seed()

   dims(1) = 1024
   dims(2) = 1024

   flatSet = dataSet("flat.fts", create)
   flatArray = addArray(flatSet, "PRIMARY", real32, dims)
   flatfieldp => real32array2data(flatArray)
   call random_number(flatfieldp)

   flatfieldp = 2.0 * flatfieldp

   flatfield = 0.0
   do i = 0, 2047
      k = floor(i / 2.0)
      do j = 0, 2047
         l = floor(j / 2.0)
         flatfield(i, j) = flatfieldp(k, l)
      end do
   end do

   x0 = 1000
   y0 = 1000
   nx = 100
   ny = 100
   binax1 = 0
   binax2 = 0
   twixel = .false.
   dims(1) = nx
   dims(2) = ny
   outSet = dataSet("image.fts", create)
   outArray = addArray(outSet, "PRIMARY", real32, dims)
   qualArray = addArray(outSet, "QUALITY", integer8, dims)
   qual => int8array2data(qualArray)
   qual = 0

   image => real32array2data(outArray)

   image = 100.0

   thxDataSetH = dataSet("trhist.safe", read)

   thxTableH = table(thxDataSetH, "OMTHX1")

   nFrames = numberOfRows(thxTableH)

   allocate (tf(0:nFrames), stat=allocErr)
   if (allocErr /= 0) then
      write (*, *) "Can't allocate memory for internal array"
!      call fatal("allocError", "Can't allocate memory for internal array")
   end if

   thxColumnH(1) = column(thxTableH, "DX", read)
   dXP => int32Data(thxColumnH(1))
   tf(0)%dX = 0.0
   tf(1:nFrames)%dX = real(dXP / 1000.0)

   thxColumnH(2) = column(thxTableH, "DY", read)
   dYP => int32Data(thxColumnH(2))
   tf(0)%dY = 0.0
   tf(1:nFrames)%dY = real(dYP / 1000.0)

   call release(thxColumnH(1))
   call release(thxColumnH(2))
!   call release(thxColumnH(3))
   call release(thxTableH)
   call release(thxDataSetH)

   allocate(newimage(0:nx-1,0:ny-1))

   newimage = 0.0
   do i = 1 , nFrames
      rShiftX = tf(i)%dx - tf(i-1)%dx
      rShiftY = tf(i)%dy - tf(i-1)%dy
      do j = 0, 10
         iShiftX = nint(0.5 + tf(i-1)%dx + (rShiftX * (j / 10.0)))
         iShiftY = nint(0.5 + tf(i-1)%dy + (rShiftY * (j / 10.0)))
         iXMin = (x0) - iShiftX
         iXMax = (x0 + nx - 1) - iShiftX
         iYMin = (y0) - iShiftY
         iYMax = (y0 + ny - 1) - iShiftY
         newimage = newimage + (image * flatfield(iXMin:iXMax, iYMin:iYMax) / 11.0)
      end do
   end do

   newimage = newimage / real(nFrames)

   image = newimage

   call setAttribute(outSet,"DATAMODE","IMAGING")
   call setAttribute(outSet,"OBS_ID","1")   
   call setAttribute(outSet,"EXP_ID","1")   
   call setAttribute(outSet,"FILTER","B")  
   call setAttribute(outSet,"BINBPE",.false.)
   call setAttribute(outSet,"BINAX1",binax1)
   call setAttribute(outSet,"BINAX2",binax2)

   outTable = addTable(outSet, "MODES", 1)

   pcol = addColumn(outTable, "WINDOWX0", integer32)
   icol => int32data(pcol)
   icol(0) = x0
   call release(pcol)

   pcol = addColumn(outTable, "WINDOWY0", integer32)
   icol => int32data(pcol)
   icol(0) = y0
   call release(pcol)

   pcol = addColumn(outTable, "WINDOWDX", integer32)
   icol => int32data(pcol)
   icol(0) = nx
   call release(pcol)

   pcol = addColumn(outTable, "WINDOWDY", integer32)
   icol => int32data(pcol)
   icol(0) = ny
   call release(pcol)

   call release(flatArray)
   call release(flatSet)
   call release(outTable)
   call release(outArray)
   call release(qualArray)
   call release(outSet)

end program gentest
